#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import logging
logger = logging.getLogger('PyAMS (cache)')

# import interfaces
from pyams_cache.interfaces import ICacheHandler, IAioCacheHandler

# import packages
from pyams_utils.registry import utility_config


# Default Redis handler
try:
    from redis import StrictRedis
except ImportError:
    logger.debug("Missing redis package. Can't init Redis cache handler.")
else:
    @utility_config(name='redis', provides=ICacheHandler)
    class RedisCacheHandler(object):
        """Redis cache handler utility"""

        client = None

        def open(self, server):
            host, port = server.split(':', 1)
            self.client = StrictRedis(host=host, port=int(port))

        def get(self, key, default=None):
            result = self.client.get(key)
            if result is None:
                result = default
            return result

        def set(self, key, value):
            self.client.set(key, value)


# AsyncIO Redis handler
try:
    import aioredis
except ImportError:
    logger.debug("Missing aioredis package. Can't init Redis asyncio cache handler.")
else:
    @utility_config(name='redis', provides=IAioCacheHandler)
    class RedisAioCacheHandler(object):
        """Redis asyncio cache handler utility"""

        connection = None

        def open(self, server):
            ip, port = server.split(':')
            self.connection = ip, int(port)

        def get(self, key, default=None):
            connection = yield from aioredis.create_connection(self.connection)
            yield from connection.execute('get', key)

        def set(self, key, value):
            connection = yield from aioredis.create_connection(self.connection)
            yield from connection.execute('set', key, value)
